﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CommonLib;
using System.Diagnostics;
using DxInput;
using CommonLib.Control;
using System.Windows.Forms;
using SimpleHID;
using System.Runtime.InteropServices;
using System.Threading;

namespace DllControlDeviceMan
{
    class SPC_7000
    {
        
        public SPC_7000()
        {
            ConnectDevice();
            CurrentMode = CONTROLLER_MODE.CONTROL;
        }

        public void ConnectDevice()
        {
            var hidInfoSet = SimpleHID.HIDManager.GetInfoSets();

            foreach (HIDInfoSet hidInfo in hidInfoSet)
            {
                if (hidInfo.ProductString == "SPC6001" || hidInfo.ProductString == "SPC7000")
                {
                    m_Spc7000JoyStick = new HIDDevice();
                    if (m_Spc7000JoyStick.Init(hidInfo.DevicePath) == true)
                    {
                        if (m_Spc7000JoyStick.RecvUSBInputMsgEvtHandler == null)
                        {
                            m_Spc7000JoyStick.RecvUSBInputMsgEvtHandler += new EventHandler(RecvUSBMsg);
                        }
                        m_bDeviceConnected = true;
                    }
                    break;
                }
            }

            if (m_Spc7000JoyStick != null && m_bDeviceConnected == true)
            {
                m_Spc7000JoyStick.BeginAsyncRead();
                m_JoystickThread = null;
                m_JoystickThread = new System.Threading.Thread(this.ProcessPanTiltZoomJogData);
                m_JoystickThread.Start();
            }
        }

        public bool IsConnected()
        {
            if (m_Spc7000JoyStick != null)
            {
                var hidInfoSet = SimpleHID.HIDManager.GetInfoSets();
                bool bConnected = false;
                foreach (HIDInfoSet hidInfo in hidInfoSet)
                {
                    if (hidInfo.ProductString == "SPC6001" || hidInfo.ProductString == "SPC7000")
                    {
                        bConnected = true;
                        break;
                    }
                }

                return bConnected;
            }
            else
            {
                return false;
            }
        }

        ~SPC_7000()
        {
        }

        public void Dispose()
        {
            if (m_Spc7000JoyStick != null)
            {
                m_Spc7000JoyStick.Dispose();
                m_Spc7000JoyStick.RecvUSBInputMsgEvtHandler -= new EventHandler(RecvUSBMsg);
                m_bDeviceConnected = false;
                m_Spc7000JoyStick = null;
            }
        }

        object locker = new object();
        public void ProcessPanTiltZoomJogData()
        {
            while (!m_bExit)
            {
                try
                {
                    // handle joystick value (JoystickPos value is joystick x,y,z)
                }
                catch (Exception e)
                {

                }
            }
        }

        public void ReadThreadStop()
        {
            try
            {
                if (m_JoystickThread != null)
                {
                    m_bExit = true;

                    if (!m_JoystickThread.Join(5000))
                    {
                        if (m_JoystickThread.IsAlive)
                        {
                            m_JoystickThread.Abort();
                            m_JoystickThread.Join();
                        }
                    }

                    m_JoystickThread = null;
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine(string.Format("[SPC_7000 ReadThreadStop]Exception : {0}", e.Message.ToString()));
            }
        }

        bool bXYZChanged = false;
        bool m_bShuttleMoved = false;
        KeyDefine m_PreKeyDeinfe = KeyDefine.NONE;
        bool bFirstMove = false;
        void RecvUSBMsg(object sender, EventArgs e)
        {
            byte[] recvData = (byte[])sender;
            KeyDefine inputKey = KeyDefine.NONE;
            nKeyDataOffset = -1;

            for (int i = 0; i < recvData.Length; i++)
            {
                if (recvData[i] != 0)
                {
                    nKeyDataOffset = i;
                }

                if (recvData[X_KEY_PRESSED] == 0 && recvData[Y_KEY_PRESSED] == 0 && recvData[Z_KEY_PRESSED] == 0)
                {
                    if (bXYZChanged == true)
                    {
                        bXYZChanged = false;
                        bFirstMove = false;
                        JoystickPos.X = 0;
                        JoystickPos.Y = 0;
                        JoystickPos.Z = 0;
                    }
                }
            }

            if (recvData[X_KEY_PRESSED] != 0 || recvData[Y_KEY_PRESSED] != 0 || recvData[Z_KEY_PRESSED] != 0)
            {
                bXYZChanged = true;
                int nStart = X_KEY_PRESSED;
                int End = Z_KEY_PRESSED + 1;
                for (int i = nStart; i < End; i++)
                {
                    int nValue = recvData[i];
                    if (nValue > 1 && nValue < 65)
                    {
                        nValue = nValue - 8;
                        if (nValue <= 0)
                            nValue = 0;

                        float fvalue = (int)(((float)nValue / (float)56) * (float)100);
                        nValue = (int)Math.Round(fvalue);
                    }
                    else if (nValue > 191 && nValue < 256)
                    {
                        nValue = nValue - 256;
                        if (nValue <= -8)
                        {
                            nValue += 8;
                        }
                        else
                        {
                            nValue = 0;
                        }
                     
                        float fvalue = (int)(((float)nValue / (float)56) * (float)100);
                        nValue = (int)Math.Round(fvalue);
                    }

                    if (i == X_KEY_PRESSED)
                    {
                        JoystickPos.X = nValue;
                    }

                    if (i == Y_KEY_PRESSED)
                    {
                        JoystickPos.Y = nValue;
                    }

                    if (i == Z_KEY_PRESSED)
                    {
                        JoystickPos.Z = nValue;
                    }
                }
            }

            if (nKeyDataOffset != -1)
            {
                if (nKeyDataOffset == SHUTTLE)
                {
                    m_bShuttleMoved = true;
                    int nValue = recvData[nKeyDataOffset];
                    if (nValue > 247 && nValue < 256)
                    {
                        nValue = nValue - 256;
                    }

                    //Do Shuttle 
                }

                if (nKeyDataOffset == JOG)
                { // +,-
                    int nValue = recvData[nKeyDataOffset];
                    if (nValue == 255) nValue = -1;
                   // Do Jog
                }
                else
                {
                    if (recvData[nKeyDataOffset] == 1)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.LAYOUT;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.NUMBER_7;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.CAM_MTX;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.LEFT_BUTTON;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.AF;
                        else if (nKeyDataOffset == 9)
                            inputKey = KeyDefine.LOCAL_RECORDING;
                    }
                    else if (recvData[nKeyDataOffset] == 2)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.LIVE_PLAYBACK;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.VM_MODE;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.NUMBER_5;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.ENTER;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.TRACE_PATTERN;
                        else if (nKeyDataOffset == 9)
                            inputKey = KeyDefine.PAUSE;
                    }
                    else if (recvData[nKeyDataOffset] == 4)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.FREEZE_ALARAM;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.MOUSE_MODE;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.NUMBER_0;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.RIGHT_BUTTON;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.GROUP_SCAN;
                        else if (nKeyDataOffset == 9)
                        {
                            // Joystick ???
                        }
                    }
                    else if (recvData[nKeyDataOffset] == 8)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.SHOW_CAMREA;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.NUMBER_4;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.SHIFT;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.FOCUS_NEAR_FAR;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.PRESET;
                        else if (nKeyDataOffset == 9)
                            inputKey = KeyDefine.SHAPSHOT;
                    }
                    else if (recvData[nKeyDataOffset] == 16)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.NUMBER_1;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.NUMBER_9;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.UP_BUTTON;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.MOUSE_LEFT_DOWN;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.PTZ_MODE;
                        else if (nKeyDataOffset == 9)
                            inputKey = KeyDefine.RESET_EVENT_STATUS;
                    }
                    else if (recvData[nKeyDataOffset] == 32)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.NUMBER_6;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.CONSOLE_MODE;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.DOWN_BUTTON;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.ZOOM;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.FORWARD;
                    }
                    else if (recvData[nKeyDataOffset] == 64)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.RECORDER_SSM;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.NUMBER_3;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.ESC;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.MOUSE_RIGHT_DOWN;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.STOP;
                    }
                    else if (recvData[nKeyDataOffset] == 128)
                    {
                        if (nKeyDataOffset == 4)
                            inputKey = KeyDefine.NUMBER_2;
                        else if (nKeyDataOffset == 5)
                            inputKey = KeyDefine.NUMBER_8;
                        else if (nKeyDataOffset == 6)
                            inputKey = KeyDefine.FAVORITE;
                        else if (nKeyDataOffset == 7)
                            inputKey = KeyDefine.IRIS_CLOSE_OPEN;
                        else if (nKeyDataOffset == 8)
                            inputKey = KeyDefine.BACKWARD;
                    }

                    m_PreKeyDeinfe = inputKey;
                    //Notice Key Input
                }
            }
            else
            {
                if (m_bShuttleMoved == true)
                {
                    m_bShuttleMoved = false;
                   // Stop Shuttle
                }
            }
        }

        public void Reset_Joystick()
        {
            //HandlePTZ(CONTROLLER_COMMAND.PTZ_MOVE_SYNC, null);
            //HandlePTZ(CONTROLLER_COMMAND.PTZ_STOP, null);
            m_bJoystick_Moved = false;
        }

        public delegate Int32 DelegateEvent(EventObj EvtObj);
        public DelegateEvent m_deleEvent = null;

        public enum KeyDefine
        {
            LAYOUT = 0, // Layout(seq.)
            LIVE_PLAYBACK, // Search
            FREEZE_ALARAM,
            SHOW_CAMREA, // CAMERA
            NUMBER_1,
            NUMBER_6,
            RECORDER_SSM,
            NUMBER_2,
            NUMBER_7,
            VM_MODE, // TILE(VM)
            MOUSE_MODE, // MENU(Mouse)
            NUMBER_4,
            NUMBER_9,
            CONSOLE_MODE, // Mon.(Con.)
            NUMBER_3,
            NUMBER_8,
            CAM_MTX,
            NUMBER_5,
            NUMBER_0,
            SHIFT,
            UP_BUTTON, // Up
            DOWN_BUTTON, // Down
            ESC, // ESC(Setup)
            FAVORITE, // Favori.(Macro)
            LEFT_BUTTON,
            ENTER,
            RIGHT_BUTTON,
            FOCUS_NEAR_FAR, // FOCUS N/F
            MOUSE_LEFT_DOWN, // T.Lock(L)
            ZOOM,
            MOUSE_RIGHT_DOWN, // TRACK/R
            IRIS_CLOSE_OPEN, // IRIS C/O
            AF,
            TRACE_PATTERN,
            GROUP_SCAN,
            PRESET,
            PTZ_MODE, // PTZ_MENU
            FORWARD,
            STOP,
            BACKWARD,
            LOCAL_RECORDING,
            PLAY,
            SHAPSHOT,

            //
            RESET_EVENT_STATUS,
            PAUSE,
            SHUTTLE_WHEEL,
            JOG_WHEEL,
            NONE,
        };

        CONTROLLER_MODE CurrentMode{ get;set; }

        public bool bConnected
        {
            get { return m_bDeviceConnected; }
        }

        bool m_bJoystick_Moved = false;
        bool m_bPTZMode = true;
        bool m_bDeviceConnected = false;
        bool m_b3AXIS = false;

        HIDDevice m_Spc7000JoyStick;
        private System.Threading.Thread m_JoystickThread; // Pan,Tilt,Zoom,Jog 값을 주기적으로 전달해주는 Thread
        private bool m_bExit = false;

        readonly int X_KEY_PRESSED = 1;
        readonly int Y_KEY_PRESSED = 2;
        readonly int Z_KEY_PRESSED = 3;
        readonly int JOG = 10;
        readonly int SHUTTLE = 11;
        public struct JoyStickPoistion
        {
            public int X;
            public int Y;
            public int Z;
        }
        JoyStickPoistion JoystickPos = new JoyStickPoistion();
        int nKeyDataOffset = 4;
        object numlocker = new object();
    }
}
